/*
 * Logswan 2.1.8
 * Copyright (c) 2015-2020, Frederic Cambus
 * https://www.logswan.org
 *
 * Created:      2015-05-31
 * Last Updated: 2019-10-17
 *
 * Logswan is released under the BSD 2-Clause license.
 * See LICENSE file for details.
 */

const char *methodsNames[] = {
	"GET",
	"POST",
	"HEAD",
	"OPTIONS",
	"PUT",
	"DELETE",
	"TRACE",
	"CONNECT",
	"PATCH"
};

const char *protocolsNames[] = {
	"HTTP/1.1",
	"HTTP/1.0",
	"HTTP/2.0",
	"HTTP/3"
};
