/*
 * Logswan 2.1.8
 * Copyright (c) 2015-2020, Frederic Cambus
 * https://www.logswan.org
 *
 * Created:      2015-05-31
 * Last Updated: 2019-01-19
 *
 * Logswan is released under the BSD 2-Clause license.
 * See LICENSE file for details.
 */

#ifndef CONTINENTS_H
#define CONTINENTS_H

extern char *continentsId[];
extern char *continentsNames[];

#endif /* CONTINENTS */
