/*
 * Logswan 2.1.8
 * Copyright (c) 2015-2020, Frederic Cambus
 * https://www.logswan.org
 *
 * Created:      2015-05-31
 * Last Updated: 2019-01-19
 *
 * Logswan is released under the BSD 2-Clause license.
 * See LICENSE file for details.
 */

const char *continentsId[] = {
	"AF",
	"AN",
	"AS",
	"EU",
	"NA",
	"OC",
	"SA"
};

const char *continentsNames[] = {
	"Africa",
	"Antarctica",
	"Asia",
	"Europe",
	"North America",
	"Oceania",
	"South America"
};
